package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.QuestionType;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.service.TeachcloudCommonService;
import com.qmth.teachcloud.common.util.FileStoreUtil;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkQuestionDto;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.mapper.MarkQuestionMapper;
import com.qmth.teachcloud.mark.params.MarkObjectiveQuestionParams;
import com.qmth.teachcloud.mark.params.MarkQuestionParams;
import com.qmth.teachcloud.mark.service.MarkGroupService;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 小题信息表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkQuestionServiceImpl extends ServiceImpl<MarkQuestionMapper, MarkQuestion> implements MarkQuestionService {

    @Resource
    private TeachcloudCommonService teachcloudCommonService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkGroupService markGroupService;
    @Resource
    private FileStoreUtil fileStoreUtil;

    @Override
    public String assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(Long examId, String paperNumber, Integer groupNumber) {
        return this.baseMapper.assembleQuestionsByExamIdAndPaperNumberAndNumber(examId, paperNumber, groupNumber);
    }

    @Override
    public List<MarkQuestion> listQuestionByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber)
                .orderByAsc(MarkQuestion::getMainNumber)
                .orderByAsc(MarkQuestion::getSubNumber);
        return this.list(queryWrapper);
    }

    @Transactional
    @Override
    public void saveQuestions(MarkQuestionParams markQuestionParams) {
        Long examId = markQuestionParams.getExamId();
        String paperNumber = markQuestionParams.getPaperNumber();

        List<MarkQuestion> questions = markQuestionParams.getQuestions();
        if (CollectionUtils.isEmpty(questions)) {
            throw ExceptionResultEnum.ERROR.exception("没有可保存的结构数据");
        }

        // 校验题号不能重复
        Map<String, Long> longMap = questions.stream().collect(Collectors.groupingBy(m -> String.valueOf(m.getMainNumber()) + m.getSubNumber(), Collectors.counting()));
        long count = longMap.entrySet().stream().filter(m -> m.getValue().intValue() > 1).count();
        if (count > 0) {
            throw ExceptionResultEnum.ERROR.exception("大题号+小题号有重复数据");
        }

        // 查询已有结构信息
        List<MarkQuestion> markQuestionList = this.listQuestionByExamIdAndPaperNumber(examId, paperNumber);
        if (CollectionUtils.isEmpty(markQuestionList)) {
            for (MarkQuestion question : questions) {
                if (question.getTotalScore() <= 0) {
                    throw ExceptionResultEnum.ERROR.exception("小题满分必须大于0");
                }
                if (question.getIntervalScore() <= 0) {
                    throw ExceptionResultEnum.ERROR.exception("间隔分必须大于0");
                }
                question.setExamId(examId);
                question.setPaperNumber(paperNumber);
            }
            this.saveBatch(questions);
        } else {
            List<MarkQuestion> saveOrUpdateList = new ArrayList<>();
            for (MarkQuestion question : questions) {
                if (Objects.isNull(question.getId())) {
                    saveOrUpdateList.add(question);
                } else {
                    Optional<MarkQuestion> questionOptional = markQuestionList.stream().filter(m -> m.getId().equals(question.getId())).findFirst();
                    if (questionOptional.isPresent()) {
                        MarkQuestion markQuestion = questionOptional.get();
                        if (markQuestion.getGroupNumber() != null && markQuestion.getGroupNumber().intValue() > 0 && markQuestion.getTotalScore().doubleValue() - question.getTotalScore().doubleValue() != 0) {
                            throw ExceptionResultEnum.ERROR.exception("大题号" + markQuestion.getMainNumber() + "小题号" + markQuestion.getSubNumber() + "已分组，不能修改小题满分");
                        }
                        if (question.getTotalScore() <= 0) {
                            throw ExceptionResultEnum.ERROR.exception("小题满分必须大于0");
                        }
                        if (!QuestionType.findByValue(question.getQuestionType()).getObjective() && question.getIntervalScore() <= 0) {
                            throw ExceptionResultEnum.ERROR.exception("间隔分必须大于0");
                        }
                        markQuestion.setTotalScore(question.getTotalScore());
                        markQuestion.setIntervalScore(question.getIntervalScore());
                        saveOrUpdateList.add(markQuestion);
                        markQuestionList.remove(markQuestion);
                    } else {
                        throw ExceptionResultEnum.ERROR.exception("数据异常，请刷新页面后重新操作");
                    }
                }
            }
            // 删除的结构
            if (CollectionUtils.isNotEmpty(markQuestionList)) {
                List<Long> deleteIds = markQuestionList.stream().map(MarkQuestion::getId).collect(Collectors.toList());
                this.removeByIds(deleteIds);
            }
            this.saveOrUpdateBatch(saveOrUpdateList);
        }
        // 更新客观题满分、主观题满分、总分
        List<MarkQuestion> markQuestions = this.listQuestionByExamIdAndPaperNumber(examId, paperNumber);
        Double objectiveScore = markQuestions.stream().filter(m -> m.getObjective()).collect(Collectors.summingDouble(m -> m.getTotalScore()));
        Double subjectiveScore = markQuestions.stream().filter(m -> !m.getObjective()).collect(Collectors.summingDouble(m -> m.getTotalScore()));
        Double totalScore = markQuestions.stream().collect(Collectors.summingDouble(m -> m.getTotalScore()));
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getObjectiveScore, objectiveScore)
                .set(MarkPaper::getSubjectiveScore, subjectiveScore)
                .set(MarkPaper::getTotalScore, totalScore)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        markPaperService.update(updateWrapper);

        // 修改分组题目总分
        Map<Integer, Double> collectGroupScoreMap = markQuestions.stream().filter(m -> m.getGroupNumber() != null).collect(Collectors.groupingBy(m -> m.getGroupNumber(), Collectors.summingDouble(m -> m.getTotalScore())));
        if (MapUtils.isNotEmpty(collectGroupScoreMap)) {
            for (Map.Entry<Integer, Double> entry : collectGroupScoreMap.entrySet()) {
                markGroupService.updateTotalScore(examId, paperNumber, entry.getKey(), entry.getValue());
            }
        }
    }

    @Override
    public List<MarkQuestion> listQuestionByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer number, Boolean isObjective) {
        LambdaQueryWrapper<MarkQuestion> lambdaQueryWrapper = new QueryWrapper<MarkQuestion>().lambda();
        lambdaQueryWrapper.eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber);
        if (number != null) {
            lambdaQueryWrapper.eq(MarkQuestion::getGroupNumber, number);
        }
        if (isObjective != null) {
            lambdaQueryWrapper.eq(MarkQuestion::getObjective, isObjective);
        }
        lambdaQueryWrapper.orderByAsc(MarkQuestion::getMainNumber)
                .orderByAsc(MarkQuestion::getSubNumber);
        return this.list(lambdaQueryWrapper);
    }

    @Transactional
    @Override
    public void saveObjectiveQuestions(MarkObjectiveQuestionParams markObjectiveQuestionParams) {
        for (MarkQuestion question : markObjectiveQuestionParams.getObjectiveInfo()) {
            if (StringUtils.isBlank(question.getAnswer())) {
                throw ExceptionResultEnum.ERROR.exception("答案不能为空");
            }
            UpdateWrapper<MarkQuestion> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(MarkQuestion::getAnswer, question.getAnswer());
            if (question.getObjectivePolicy() != null) {
                updateWrapper.lambda().set(MarkQuestion::getObjectivePolicy, question.getObjectivePolicy());
            }
            updateWrapper.lambda().eq(MarkQuestion::getId, question.getId());
            this.update(updateWrapper);
        }
    }

    @Override
    public String uploadSubjectiveFile(Long examId, String paperNumber, MultipartFile file, String md5) {
        String answerFilePath = fileStoreUtil.uploadFile(file, md5, UploadFileEnum.PDF);
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getAnswerFilePath, answerFilePath)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        markPaperService.update(updateWrapper);
        return teachcloudCommonService.filePreview(answerFilePath);
    }

    @Override
    public String previewAnswerFileByExamIdAndPaperNumber(Long examId, String paperNumber) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        if (markPaper != null) {
            return teachcloudCommonService.filePreview(markPaper.getAnswerFilePath());
        }
        return null;
    }

    @Override
    public double sumTotalScoreByGroupNumber(Long examId, String paperNumber, Integer groupNumber) {
        Double score = this.baseMapper.sumTotalScoreByGroupNumber(examId, paperNumber, groupNumber);
        return score != null ? score.doubleValue() : 0d;
    }

    @Override
    public long countQuestionByExamIdAndPaperNumberAndGroupNumberIsNull(Long examId, String paperNumber, boolean isObjective) {
        QueryWrapper<MarkQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber)
                .eq(MarkQuestion::getObjective, isObjective)
                .isNull(MarkQuestion::getGroupNumber);
        return this.count(queryWrapper);
    }

    @Override
    public MarkQuestion getByExamIdAndPaperNumberAndMainNumberAndSubNumberAndObjective(Long examId, String paperNumber, Integer mainNumber, Integer subNumber, boolean isObjective) {
        QueryWrapper<MarkQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber)
                .eq(MarkQuestion::getMainNumber, mainNumber)
                .eq(MarkQuestion::getSubNumber, subNumber)
                .eq(MarkQuestion::getObjective, isObjective);
        return this.getOne(queryWrapper);
    }

    @Override
    public List<MarkQuestion> listByExamIdAndPaperNumberAndPaperIndexAndPageIndex(Long examId, String paperNumber,
                                                                                  Integer paperIndex, Integer pageIndex) {
        QueryWrapper<MarkQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber)
                .eq(MarkQuestion::getPaperIndex, paperIndex)
                .eq(MarkQuestion::getPageIndex, pageIndex);
        return this.list(queryWrapper);
    }

    @Override
    public MarkQuestionDto pageQuestionsByExamIdAndPaperNumber(Long examId, String paperNumber, String paperType, boolean canCreate) {
        MarkQuestionDto markQuestionDto = new MarkQuestionDto();
        markQuestionDto.setCanCreate(canCreate);
        List<MarkQuestion> markQuestionList = this.listQuestionByExamIdAndPaperNumber(examId, paperNumber);
        long subjectiveCount = markQuestionList.stream().filter(m -> !m.getObjective() && m.getTotalScore() != null).count();
        markQuestionDto.setQuestionSubmit(subjectiveCount > 0);
        long objectiveCount = markQuestionList.stream().filter(m -> m.getObjective() && StringUtils.isNotBlank(m.getAnswer())).count();
        markQuestionDto.setObjectiveSubmit(objectiveCount > 0);
        markQuestionDto.setQuestions(markQuestionList);
        return markQuestionDto;
    }

    @Override
    public void updateGroupNumberByExamIdAndPaperNumberAndGroupNumber(Integer newGroupNumber, Long examId, String paperNumber, Integer groupNumber) {
        UpdateWrapper<MarkQuestion> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkQuestion::getGroupNumber, newGroupNumber)
                .eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber)
                .eq(MarkQuestion::getGroupNumber, groupNumber);
        this.update(updateWrapper);
    }

    @Override
    public long countByExamIdAndPaperNumberAndObjectiveAndGroupNumberIsNull(Long examId, String paperNumber, boolean objective) {
        QueryWrapper<MarkQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber)
                .eq(MarkQuestion::getObjective, objective)
                .isNull(MarkQuestion::getGroupNumber);
        return this.count(queryWrapper);
    }

    @Override
    public List<MarkQuestion> listByExamIdAndPaperNumberAndPaperType(Long examId, String paperNumber, String paperType, Boolean objective) {
        QueryWrapper<MarkQuestion> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkQuestion> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber)
                .eq(MarkQuestion::getPaperType, paperType);
        if (objective != null) {
            lambdaQueryWrapper.eq(MarkQuestion::getObjective, objective);
        }
        return this.list(queryWrapper);
    }

    @Override
    public void deleteByExamIdAndPaperNumber(Long examId, String paperNumber) {
        UpdateWrapper<MarkQuestion> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber);
        this.remove(updateWrapper);
    }

    @Override
    public List<MarkQuestion> listQuestionByExamIdAndPaperNumber(Long examId, String paperNumber, Boolean isObjective) {
        QueryWrapper<MarkQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber)
                .eq(MarkQuestion::getObjective, isObjective)
                .orderByAsc(MarkQuestion::getMainNumber)
                .orderByAsc(MarkQuestion::getSubNumber);
        return this.list(queryWrapper);
    }
}
